const UserAccountDetailsRepository = require('../repositories/userAccountDetailsRepository');
const UserAccountsRepository = require('../repositories/userAccountsRepository');
const logger = require('../logger/logger');

const createAccountDetails = async (request, response) => {
    const { first, last, address1, address2, city, usstate, zip, phone, email, userId } = request.body;
    // Checks if user account exists
    const userAccountsRepository = new UserAccountsRepository();
    const existingUserAccount = await userAccountsRepository.selectById(userId);
    if (existingUserAccount) {
        // Checks if user account details exists
        const userAccountDetailsRepository = new UserAccountDetailsRepository();
        const existingUserAccountDetails = await userAccountDetailsRepository.select(userId);
        if (existingUserAccountDetails == 0) {
            logger.info('Creating new record in user_account_details table');
            await userAccountDetailsRepository.insert(first, last, address1, address2, city, usstate, zip, phone, email, userId);
            return response.status(200).json(request.body);
        } else {
            const error = `Details for account ID ${userId} already exists`;
            logger.error(error);
            return response.status(409).json({ error });
        }
    } else {
        const error = `${userId} is not an active account's ID`;
        logger.error(error);
        return response.status(409).json({ error });
    }
};

const fetchAccountDetails = async (request, response) => {
    const { id } = request.query;
    const userAccountDetailsRepository = new UserAccountDetailsRepository();
    const existingUserAccountDetails = await userAccountDetailsRepository.select(id);
    if (existingUserAccountDetails.length > 0) {
        return response.status(200).json({
            userDetails: existingUserAccountDetails.map(({ first_name, last_name, address_1, address_2, state, zip_code, phone_number, ...existingUserAccountDetails }) =>
                ({ first: first_name, last: last_name, address1: address_1, address2: address_2, usstate: state, zip: zip_code, phone: phone_number, ...existingUserAccountDetails }))
        });
    } else {
        const error = `user ${id} does not have any account details on record`;
        logger.error(error);
        return response.status(204).json({ error });
    }
}

const updateAccountDetails = async (request, response) => {
    const { first, last, address1, address2, city, usstate, zip, phone, email, userId } = request.body;
    // Checks if user account details exists
    const userAccountDetailsRepository = new UserAccountDetailsRepository();
    const existingUserAccount = await userAccountDetailsRepository.select(userId);
    if (existingUserAccount.length > 0) {
        logger.info('Updating record in user_account_details table');
        await userAccountDetailsRepository.update(first, last, address1, address2, city, usstate, zip, phone, email, userId);
        return response.status(200).json(request.body);
    } else {
        const error = `User ID ${userId} does not have any account details`;
        logger.error(error);
        return response.status(404).json({ error });
    }
}

module.exports = {createAccountDetails: createAccountDetails, fetchAccountDetails: fetchAccountDetails, updateAccountDetails: updateAccountDetails};